@@ -36,7 +36,10 @@ def plot(ts, title=None, show=True):
         for i in range(num_subplots):
             ax = fig.add_subplot(num_subplots, 1, i+1)
             ax.plot(ts.tspan, ts[...,i])
-            ax.set_ylabel('node ' + str(i), **ylabelprops)
+            if ts.labels[-1] is not None:
+                ax.set_ylabel(ts.labels[-1][i], **ylabelprops)
+            else:
+                ax.set_ylabel('node ' + str(i), **ylabelprops)
             plt.setp(ax.get_xticklabels(), visible=False)
         fig.axes[0].set_title(title)
         plt.setp(fig.axes[num_subplots-1].get_xticklabels(), visible=True)
@@ -54,7 +57,7 @@ def plot(ts, title=None, show=True):
             rect = 0.1, 0.85*(num_ax - i - 1)/num_ax + 0.1, 0.8, 0.85/num_ax
             ax = fig.add_axes(rect, **axprops)
             # use i'th color in cycle
-            _ = [ax._get_lines.color_cycle.next() for j in range(i)]
+            _ = [next(ax._get_lines.color_cycle) for j in range(i)]
             ax.plot(ts.tspan, ts[...,i])
             plt.setp(ax.get_xticklabels(), visible=False)
             if ts.labels[1] is not None:
