@@ -28,7 +28,7 @@ def plot(dts, title=None, points=None, show=True):
     """
     if points is not None and len(dts.tspan) > points:
         # then downsample  (TODO: use interpolation)
-        ix = np.linspace(0, len(dts.tspan) - 1, points, dtype=np.int64)
+        ix = np.linspace(0, len(dts.tspan) - 1, points).astype(np.int64)
         dts = dts[ix, ...]
     ts = distob.gather(dts)
     return ts.plot(title, show)
@@ -44,7 +44,7 @@ def phase_histogram(dts, times, nbins=30, colormap=mpl.cm.Blues):
       nbins (int): number of histogram bins
       colormap
     """
-    interval = dts.periods_all().mean()
+    interval = dts.periods().mean()
     snapshots = dts.t[0.0::interval].mod2pi()
     snapshots = distob.gather(snapshots)
     if isinstance(times, numbers.Number):
