@@ -42,6 +42,7 @@ import numpy as np
 from collections import Sequence
 import copy
 import types
+import warnings
 import numbers
 import random
 #from memory_profiler import profile
@@ -71,9 +72,30 @@ class RemoteTimeseries(distob.RemoteArray, object):
     """Local object representing a Timeseries that may be local or remote"""
 
     def __repr__(self):
-        return self._cached_apply('__repr__').replace(
+        return distob.methodcall(self, '__repr__').replace(
             self._ref.type.__name__, self.__class__.__name__, 1)
 
+    def __numpy_ufunc__(self, ufunc, method, i, inputs, **kwargs):
+        out_arr = super(RemoteTimeseries, self).__numpy_ufunc__(
+                ufunc, method, i, inputs, **kwargs)
+        return _ufunc_wrap(out_arr, ufunc, method, i, inputs, **kwargs)
+
+    __array_priority__ = 3.0
+
+    def __array_prepare__(self, out_arr, context=None):
+        """Fetch underlying data to user's computer and apply ufunc locally.
+        Only used as a fallback, for numpy versions < 1.10.0 which lack 
+        support for the __numpy_ufunc__ mechanism. 
+        """
+        #print('RemoteTimeseries __array_prepare__ context=%s' % repr(context))
+        out_arr = super(RemoteTimeseries, self).__array_prepare__(
+                out_arr, context)
+        if context is None:
+            return out_arr
+        else:
+            ufunc, inputs, i = context
+            return _ufunc_wrap(out_arr, ufunc, None, i, inputs)
+
     def plot(self, title=None, show=True):
         self._fetch()
         return self._obcache.plot(title, show)
@@ -84,7 +106,7 @@ class RemoteTimeseries(distob.RemoteArray, object):
           plot (bool)
         """
         if not plot:
-            return self._cached_apply('psd', plot=False)
+            return distob.methodcall(self, 'psd', plot=False)
         else:
             self._fetch()
             return self._obcache.psd(plot=True)
@@ -188,16 +210,46 @@ class DistTimeseries(distob.DistArray):
                 not all(t == self.tspan[-1] for t in ends)):
             raise SimValueError(u'timeseries must use the same time points')
         nlabels = [rts.labels for rts in self._subarrays]
-        self.labels = [None]
+        self.labels = [None] * self.ndim
         # only give labels to an axis if all sub-timeseries agree
-        for i in range(1, self.ndim - 1):
-            if len(set(labels[i] for labels in nlabels)) is 1:
-                self.labels.append(nlabels[0][i])
+        for i in range(1, self.ndim):
+            if i is self._distaxis:
+                self.labels[i] = axislabels
             else:
-                self.labels.append(None)
-        self.labels.insert(self._distaxis, axislabels)
+                candidate = nlabels[0][i]
+                if all(labels[i] == candidate for labels in nlabels[1:]):
+                    self.labels[i] = candidate
+        if self._obcache_current:
+            self._obcache.labels[self._distaxis] = axislabels
         self.t = _Timeslice(self)
 
+    def _fetch(self):
+        """forces update of a local cached copy of the real object
+        (regardless of the preference setting self.cache)"""
+        if not self._obcache_current:
+            from distob import engine
+            ax = self._distaxis
+            self._obcache = distob.concatenate(
+                    [ra._ob for ra in self._subarrays], ax)
+            if hasattr(self, 'labels'):
+                self._obcache.labels[ax] = self.labels[ax]
+            # let subarray obcaches and main obcache be views on same memory:
+            for i in range(self._n):
+                ix = [slice(None)] * self.ndim
+                ix[ax] = slice(i, i+1)
+                self._subarrays[i]._obcache = self._obcache[tuple(ix)]
+            self._obcache_current = True
+            # now prefer local processing:
+            self.__engine_affinity__ = (
+                    engine.eid, self.__engine_affinity__[1])
+
+    def __ob(self):
+        """return a copy of the real object"""
+        self._fetch()
+        return self._obcache
+
+    _ob = property(fget=__ob, doc='return a local copy of the object')
+
     @classmethod
     def add_analyses(cls, source, vectorize=False):
         """Dynamically add new analysis methods to the DistTimeseries class.
@@ -245,12 +297,12 @@ class DistTimeseries(distob.DistArray):
     def __getitem__(self, index):
         """Slice the distributed timeseries"""
         ar = super(DistTimeseries, self).__getitem__(index)
-        if (isinstance(ar, RemoteTimeseries) or 
-                isinstance(ar, distob.RemoteArray)):
+        if isinstance(ar, distob.RemoteArray) or isinstance(ar, np.ndarray):
             # slicing result is no longer distributed
             return ar
         # otherwise `ar` is a DistArray
-        if not isinstance(ar._subarrays[0], RemoteTimeseries):
+        if (not isinstance(ar._subarrays[0], RemoteTimeseries) and
+                not isinstance(ar._subarrays[0], Timeseries)):
             return ar
         cur_labels = self.labels
         cur_shape = self.shape # current values, may be updated by np.newaxis
@@ -291,7 +343,9 @@ class DistTimeseries(distob.DistArray):
             raise IndexError('too many indices for array')
         if basic_slicing:
             new_tspan = self.tspan[index[0]]
-            if not isinstance(new_tspan, np.ndarray) or new_tspan.shape is ():
+            if (not isinstance(new_tspan, np.ndarray) or 
+                    new_tspan.shape is () or
+                    new_tspan.shape[0] is 0):
                 # axis 0 has been sliced away: result is no longer a Timeseries
                 return ar
             new_labels = [None]
@@ -331,7 +385,8 @@ class DistTimeseries(distob.DistArray):
             if contiguous and not is_fancy[0]:
                 new_tspan = self.tspan[index[0]]
                 if (not isinstance(new_tspan, np.ndarray) or
-                        new_tspan.shape is ()):
+                        new_tspan.shape is () or
+                        new_tspan.shape[0] is 0):
                     # axis 0 has been sliced away: no longer a Timeseries
                     return ar
             # compute labels for the nonfancy output axes
@@ -395,7 +450,8 @@ class DistTimeseries(distob.DistArray):
                         if label_source is 0:
                             new_tspan = self.tspan[index[0][iix]]
                             if (not isinstance(new_tspan, np.ndarray) or
-                                    new_tspan.shape is ()):
+                                    new_tspan.shape is () or
+                                    new_tspan.shape[0] is 0):
                                 # axis 0 has been sliced away: not a Timeseries
                                 return ar
                             if not np.all(np.diff(new_tspan) > 0):
@@ -454,6 +510,27 @@ class DistTimeseries(distob.DistArray):
             labelsrep = ', \nlabels=' + repr(self.labels)
         return head + content + labelsrep + ')'
 
+    def __numpy_ufunc__(self, ufunc, method, i, inputs, **kwargs):
+        out_arr = super(DistTimeseries, self).__numpy_ufunc__(
+                ufunc, method, i, inputs, **kwargs)
+        return _ufunc_wrap(out_arr, ufunc, method, i, inputs, **kwargs)
+
+    __array_priority__ = 3.0
+
+    def __array_prepare__(self, out_arr, context=None):
+        """Fetch underlying data to user's computer and apply ufunc locally.
+        Only used as a fallback, for numpy versions < 1.10.0 which lack 
+        support for the __numpy_ufunc__ mechanism. 
+        """
+        #print('DistTimeseries __array_prepare__ context=%s' % repr(context))
+        out_arr = super(DistTimeseries, self).__array_prepare__(
+                out_arr, context)
+        if context is None:
+            return out_arr
+        else:
+            ufunc, inputs, i = context
+            return _ufunc_wrap(out_arr, ufunc, None, i, inputs)
+
     @classmethod
     def __distob_vectorize__(cls, f):
         """Upgrades a normal function f to act on a DistTimeseries in parallel
@@ -470,30 +547,20 @@ class DistTimeseries(distob.DistArray):
             not arrays, will return a list with the result for each
             sub-timeseries)
         """
+        # TODO: shares much code with the superclass method. refactor.
+        def _reduced_f(a, distaxis, *args, **kwargs):
+            """(Executed on a remote or local engine) Remove specified axis
+            from array `a` and then apply f to it"""
+            remove_axis = ((slice(None),)*(distaxis) + (0,) +
+                           (slice(None),)*(a.ndim - distaxis - 1))
+            return f(a[remove_axis], *args, **kwargs)
         def vf(self, *args, **kwargs):
-            remove_axis = ((slice(None),)*(self._distaxis) + (0,) + 
-                           (slice(None),)*(self.ndim - self._distaxis - 1))
-            refs = [ra[remove_axis]._ref for ra in self._subarrays]
-            dv = distob.engine._client[:]
-            def remote_f(object_id, *args, **kwargs):
-                result = f(distob.engine[object_id], *args, **kwargs)
-                if type(result) in distob.engine.proxy_types:
-                    return distob.Ref(result)
-                else:
-                    return result
-            results = []
-            for ref in refs:
-                dv.targets = ref.engine_id
-                ar = dv.apply_async(remote_f, ref.object_id, *args, **kwargs)
-                results.append(ar)
-            for i in range(len(results)):
-                ar = results[i]
-                ar.wait()
-                results[i] = ar.r
-                if isinstance(results[i], distob.Ref):
-                    ref = results[i]
-                    RemoteClass = distob.engine.proxy_types[ref.type]
-                    results[i] = RemoteClass(ref)
+            kwargs = kwargs.copy()
+            kwargs['block'] = False
+            kwargs['prefer_local'] = False
+            ars = [distob.call(_reduced_f, ra, self._distaxis, 
+                               *args, **kwargs) for ra in self._subarrays]
+            results = [distob.convert_result(ar) for ar in ars]
             if (all(isinstance(r, distob.RemoteArray) for r in results) and
                     all(r.shape == results[0].shape for r in results)):
                 # Then we can join the results and return a DistArray.
@@ -521,7 +588,7 @@ class DistTimeseries(distob.DistArray):
                         pass
                 return distob.DistArray(results, new_distaxis)
             else:
-                return results
+                return results  # list
         if hasattr(f, '__name__'):
             vf.__name__ = 'v' + f.__name__
             f_str = f.__name__ + '()'
@@ -556,26 +623,88 @@ class DistTimeseries(distob.DistArray):
             axislabels = self.labels[self._distaxis]
             return DistTimeseries(new_subts, new_distaxis, axislabels)
 
+    def angle(self, deg=False):
+        """Return the angle of a complex Timeseries
 
-def _rts_from_ra(ra, tspan, labels):
+        Args:
+          deg (bool, optional):
+            Return angle in degrees if True, radians if False (default).
+
+        Returns:
+          angle (Timeseries):
+            The counterclockwise angle from the positive real axis on
+            the complex plane, with dtype as numpy.float64.
+        """
+        if self.dtype.str[1] != 'c':
+            warnings.warn('angle() is intended for complex-valued timeseries',
+                          RuntimeWarning, 1)
+        da = distob.vectorize(np.angle)(self, deg)
+        return _dts_from_da(da, self.tspan, self.labels)
+
+
+def _ufunc_wrap(out_arr, ufunc, method, i, inputs, **kwargs):
+    """After using the superclass __numpy_ufunc__ to route ufunc computations 
+    on the array data, convert any resulting ndarray, RemoteArray and DistArray
+    instances into Timeseries, RemoteTimeseries and DistTimeseries instances
+    if appropriate"""
+    # Assigns tspan/labels to an axis only if inputs do not disagree on them.
+    shape = out_arr.shape
+    ndim = out_arr.ndim
+    if ndim is 0 or shape[0] is 0:
+        # not a timeseries
+        return out_arr
+    candidates = [a.tspan for a in inputs if (hasattr(a, 'tspan') and
+                                              a.shape[0] == shape[0])]
+    # Expensive to validate all tspans are the same. check start and end t
+    starts = [tspan[0] for tspan in candidates]
+    ends = [tspan[-1] for tspan in candidates]
+    if len(set(starts)) != 1 or len(set(ends)) != 1:
+        # inputs cannot agree on tspan
+        return out_arr
+    else:
+        new_tspan = candidates[0]
+    new_labels = [None]
+    for i in range(1, ndim):
+        candidates = [a.labels[i] for a in inputs if (hasattr(a, 'labels') and 
+                 a.shape[i] == shape[i] and a.labels[i] is not None)] 
+        if len(candidates) is 1:
+            new_labels.append(candidates[0])
+        elif (len(candidates) > 1 and all(labs[j] == candidates[0][j] for 
+                labs in candidates[1:] for j in range(shape[i]))):
+            new_labels.append(candidates[0])
+        else:
+            new_labels.append(None)
+    if isinstance(out_arr, np.ndarray):
+        return Timeseries(out_arr, new_tspan, new_labels)
+    elif isinstance(out_arr, distob.RemoteArray):
+        return _rts_from_ra(out_arr, new_tspan, new_labels)
+    elif (isinstance(out_arr, distob.DistArray) and
+          all(isinstance(ra, RemoteTimeseries) for ra in out_arr._subarrays)):
+        return _dts_from_da(out_arr, new_tspan, new_labels)
+    else:
+        return out_arr
+
+
+def _rts_from_ra(ra, tspan, labels, block=True):
     """construct a RemoteTimeseries from a RemoteArray"""
-    eng_id = ra._ref.engine_id
-    dv = distob.engine._client[eng_id]
-    def remote_convert(ra_id, tspan, labels):
-        from distob import engine
+    def _convert(a, tspan, labels):
         from nsim import Timeseries
-        array = engine[ra_id]
-        ts = Timeseries(array, tspan, labels)
-        return Ref(ts)
-    ref = dv.apply_sync(remote_convert, ra._ref.object_id, tspan, labels)
-    return RemoteTimeseries(ref)
-
+        return Timeseries(a, tspan, labels)
+    return distob.call(
+            _convert, ra, tspan, labels, prefer_local=False, block=block)
 
 def _dts_from_da(da, tspan, labels):
-    """construct a DistTimeseries from a DistArray whose subarrays are already
-    RemoteTimeseries 
-    """
-    assert(all(isinstance(ra, RemoteTimeseries) for ra in da._subarrays))
+    """construct a DistTimeseries from a DistArray"""
+    sublabels = labels[:]
+    new_subarrays = []
+    for i, ra in enumerate(da._subarrays):
+        if isinstance(ra, RemoteTimeseries):
+            new_subarrays.append(ra)
+        else:
+            sublabels[da._distaxis] = [labels[da._distaxis][i]]
+            new_subarrays.append(_rts_from_ra(ra, tspan, sublabels, False))
+    new_subarrays = [distob.convert_result(ar) for ar in new_subarrays]
+    da._subarrays = new_subarrays
     da.__class__ = DistTimeseries
     da.tspan = tspan
     da.labels = labels
@@ -727,9 +856,11 @@ class RemoteSimulation(distob.Remote, Simulation):
 
     def compute(self):
         """Start the computation process asynchronously"""
-        def remote_compute(sim_id):
-            distob.engine[sim_id].compute()
-        self._dv.apply_async(remote_compute, self._id)
+        from distob import methodcall
+        methodcall(self, 'compute', prefer_local=False, block=False)
+        #def remote_compute(sim_id):
+        #    distob.engine[sim_id].compute()
+        #self._dv.apply_async(remote_compute, self._id)
 
 
 class MultipleSim(object):
