@@ -163,28 +163,18 @@ class Timeseries(np.ndarray):
             import distob
             if distob.engine is not None and cls in distob.engine.proxy_types:
                 RemoteClass = distob.engine.proxy_types[cls]
-                def mk_proxy_method(method_name, doc):
-                    def method(self, *args, **kwargs):
-                        if self._obcache_current:
-                            return getattr(self._obcache, method_name)(
-                                    *args, **kwargs)
-                        else:
-                            return self._try_cached_apply(
-                                    method_name, *args, **kwargs)
-                    method.__doc__ = doc
-                    method.__name__ = method_name
-                    return method
-                remotemethod = mk_proxy_method(name, f.__doc__)
+                remotemethod = distob._make_proxy_method(name, f.__doc__)
                 setattr(RemoteClass, name, remotemethod)
                 # also update class definitions on remote engines
                 if isinstance(distob.engine, distob.ObjectHub):
                     dv = distob.engine._dv
+                    dv.targets = 'all'
                     def remote_update(name, Class, f, remotemethod):
                         setattr(Class, name, f)
                         RemoteClass = distob.engine.proxy_types[Class]
                         setattr(RemoteClass, name, remotemethod)
-                    ars = self._dv.apply(remote_uptdae, real_type, proxy_type)
-                    self._dv.wait(ars)
+                    ars = dv.apply(remote_update, real_type, proxy_type)
+                    dv.wait(ars)
                     for ar in ars:
                         if not ar.successful():
                             raise ar.r
@@ -254,7 +244,9 @@ class Timeseries(np.ndarray):
             raise IndexError('too many indices for array')
         if basic_slicing:
             new_tspan = self.tspan[index[0]]
-            if not isinstance(new_tspan, np.ndarray) or new_tspan.shape is ():
+            if (not isinstance(new_tspan, np.ndarray) or 
+                    new_tspan.shape is () or
+                    new_tspan.shape[0] is 0):
                 # axis 0 has been sliced away: result is no longer a Timeseries
                 return np.asarray(new_array)
             new_labels = [None]
@@ -291,7 +283,8 @@ class Timeseries(np.ndarray):
             if contiguous and not is_fancy[0]:
                 new_tspan = self.tspan[index[0]]
                 if (not isinstance(new_tspan, np.ndarray) or
-                        new_tspan.shape is ()):
+                        new_tspan.shape is () or
+                        new_tspan.shape[0] is 0):
                     # axis 0 has been sliced away: no longer a Timeseries
                     return np.asarray(new_array)
             # compute labels for the nonfancy output axes
@@ -355,7 +348,8 @@ class Timeseries(np.ndarray):
                         if label_source is 0:
                             new_tspan = self.tspan[index[0][iix]]
                             if (not isinstance(new_tspan, np.ndarray) or
-                                    new_tspan.shape is ()):
+                                    new_tspan.shape is () or
+                                    new_tspan.shape[0] is 0):
                                 # axis 0 has been sliced away: not a Timeseries
                                 return np.asarray(new_array)
                             if not np.all(np.diff(new_tspan) > 0):
@@ -428,19 +422,35 @@ class Timeseries(np.ndarray):
         super(Timeseries, self).__setstate__(tsstate[0])
         self.__dict__.update(tsstate[1])
 
-    def __distob_scatter__(self, axis=None):
+    def __distob_scatter__(self, axis=-1, destination=None):
         """Turn a Timeseries into a distributed timeseries"""
-        if axis is None:
-            axis = self.ndim - 1
-        if axis == 0:
-            raise ValueError(u'Currently cannot distribute the time axis')
         import distob
         from nsim import DistTimeseries
+        if axis is None:
+            return distob.distob._scatter_ndarray(self, None, destination)
+        if axis == 0:
+            raise ValueError(u'Currently cannot distribute the time axis')
+        if axis < 0:
+            axis = self.ndim + axis
+        dar = distob.distob._scatter_ndarray(self, axis, destination)
         axlabels = self.labels[axis]
-        dar = distob.distob._scatter_ndarray(self, axis)
         return DistTimeseries([rts for rts in dar._subarrays], axis, axlabels)
 
-    def angle(self, deg=0):
+    def angle(self, deg=False):
+        """Return the angle of the complex argument.
+
+        Args:
+          deg (bool, optional):
+            Return angle in degrees if True, radians if False (default).
+
+        Returns:
+          angle (Timeseries):
+            The counterclockwise angle from the positive real axis on
+            the complex plane, with dtype as numpy.float64.
+        """
+        if self.dtype.str[1] != 'c':
+            warnings.warn('angle() is intended for complex-valued timeseries',
+                          RuntimeWarning, 1)
         return Timeseries(np.angle(self, deg=deg), self.tspan, self.labels)
 
     # Some of the usual array operations on Timeseries return a plain ndarray. 
